# codng: utf-8

from django_filters.filterset import BaseFilterSet, FilterSetMetaclass, resolve_field, ALL_FIELDS
from django.db.models.fields import DateField
from django_filters import utils
from django.utils.translation import ugettext as _
from .filters import DateRangeFilter


class JmbBaseFilterSet(BaseFilterSet):

    @classmethod
    def filter_for_lookup(cls, f, lookup_type):
        # override date range lookups
        if isinstance(f, DateField) and lookup_type == 'range':
            return DateRangeFilter, {}

        # use default behavior otherwise
        return super().filter_for_lookup(f, lookup_type)


class FilterSet(JmbBaseFilterSet, metaclass=FilterSetMetaclass):

    def __init__(self, *args, **kw):

        super(FilterSet, self).__init__(*args, **kw)

        if hasattr(self, 'FILTERSET_WIDGETS'):
            for field_name, widget in self.FILTERSET_WIDGETS.items():
                self.form.fields[field_name].widget = widget

        if hasattr(self.Meta, 'labels'):
            for field_name, label in self.Meta.labels.items():
                if label:
                    self.form.fields[field_name].label = "{} {}".format(
                    _(label).capitalize(),
                        utils.verbose_lookup_expr(self.filters[field_name].lookup_expr))


def filterset_factory(model, fields=ALL_FIELDS):
    meta = type(str('Meta'), (object,), {'model': model, 'fields': fields})
    filterset = type(str('%sFilterSet' % model._meta.object_name),
                     (FilterSet,), {'Meta': meta})
    return filterset
